@@ -1,6 +1,6 @@
 /* armv8-sha256.c
  *
- * Copyright (C) 2006-2020 wolfSSL Inc.
+ * Copyright (C) 2006-2023 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
@@ -44,7 +44,15 @@
     #include <wolfcrypt/src/misc.c>
 #endif
 
+#if defined(FREESCALE_MMCAU_SHA)
+    #ifdef FREESCALE_MMCAU_CLASSIC_SHA
+        #include "cau_api.h"
+    #else
+        #include "fsl_mmcau.h"
+    #endif
+#endif
 
+#ifndef WOLFSSL_ARMASM_NO_HW_CRYPTO
 static const ALIGN32 word32 K[64] = {
     0x428A2F98L, 0x71374491L, 0xB5C0FBCFL, 0xE9B5DBA5L, 0x3956C25BL,
     0x59F111F1L, 0x923F82A4L, 0xAB1C5ED5L, 0xD807AA98L, 0x12835B01L,
@@ -60,6 +68,7 @@ static const ALIGN32 word32 K[64] = {
     0x682E6FF3L, 0x748F82EEL, 0x78A5636FL, 0x84C87814L, 0x8CC70208L,
     0x90BEFFFAL, 0xA4506CEBL, 0xBEF9A3F7L, 0xC67178F2L
 };
+#endif
 
 
 static int InitSha256(wc_Sha256* sha256)
@@ -70,6 +79,17 @@ static int InitSha256(wc_Sha256* sha256)
         return BAD_FUNC_ARG;
     }
 
+#ifdef FREESCALE_MMCAU_SHA
+    ret = wolfSSL_CryptHwMutexLock();
+    if (ret == 0) {
+    #ifdef FREESCALE_MMCAU_CLASSIC_SHA
+        cau_sha256_initialize_output(sha256->digest);
+    #else
+        MMCAU_SHA256_InitializeOutput((uint32_t*)sha256->digest);
+    #endif
+        wolfSSL_CryptHwMutexUnLock();
+    }
+#else
     sha256->digest[0] = 0x6A09E667L;
     sha256->digest[1] = 0xBB67AE85L;
     sha256->digest[2] = 0x3C6EF372L;
@@ -78,11 +98,16 @@ static int InitSha256(wc_Sha256* sha256)
     sha256->digest[5] = 0x9B05688CL;
     sha256->digest[6] = 0x1F83D9ABL;
     sha256->digest[7] = 0x5BE0CD19L;
+#endif
 
     sha256->buffLen = 0;
     sha256->loLen   = 0;
     sha256->hiLen   = 0;
 
+#ifdef WOLFSSL_HASH_FLAGS
+    sha256->flags = 0;
+#endif
+
     return ret;
 }
 
@@ -94,8 +119,192 @@ static WC_INLINE void AddLength(wc_Sha256* sha256, word32 len)
 }
 
 
+#ifndef WOLFSSL_ARMASM_NO_HW_CRYPTO
+
 #ifdef __aarch64__
 
+/* First block is in sha256->buffer and rest in data. */
+static WC_INLINE void Sha256Transform(wc_Sha256* sha256, const byte* data,
+                                      word32 numBlocks)
+{
+    word32* k = (word32*)K;
+
+    __asm__ volatile (
+    "#load leftover data\n"
+    "LD1 {v0.2d-v3.2d}, %[buffer]   \n"
+
+    "#load current digest\n"
+    "LD1 {v12.2d-v13.2d}, %[digest] \n"
+    "MOV w8, %w[blocks] \n"
+    "REV32 v0.16b, v0.16b \n"
+    "REV32 v1.16b, v1.16b \n"
+    "REV32 v2.16b, v2.16b \n"
+    "REV32 v3.16b, v3.16b \n"
+
+    "#load K values in \n"
+    "LD1 {v16.4s-v19.4s}, [%[k]], #64    \n"
+    "LD1 {v20.4s-v23.4s}, [%[k]], #64    \n"
+    "MOV v14.16b, v12.16b \n" /* store digest for add at the end */
+    "MOV v15.16b, v13.16b \n"
+    "LD1 {v24.4s-v27.4s}, [%[k]], #64    \n"
+    "LD1 {v28.4s-v31.4s}, [%[k]], #64    \n"
+
+    /* beginning of SHA256 block operation */
+    "1:\n"
+    /* Round 1 */
+    "MOV v4.16b, v0.16b        \n"
+    "ADD v0.4s, v0.4s, v16.4s  \n"
+    "MOV v11.16b, v12.16b      \n"
+    "SHA256H q12, q13, v0.4s   \n"
+    "SHA256H2 q13, q11, v0.4s  \n"
+
+    /* Round 2 */
+    "SHA256SU0 v4.4s, v1.4s        \n"
+    "ADD v0.4s, v1.4s, v17.4s  \n"
+    "MOV v11.16b, v12.16b      \n"
+    "SHA256SU1 v4.4s, v2.4s, v3.4s \n"
+    "SHA256H q12, q13, v0.4s   \n"
+    "SHA256H2 q13, q11, v0.4s  \n"
+
+    /* Round 3 */
+    "SHA256SU0 v1.4s, v2.4s        \n"
+    "ADD v0.4s, v2.4s, v18.4s  \n"
+    "MOV v11.16b, v12.16b      \n"
+    "SHA256SU1 v1.4s, v3.4s, v4.4s \n"
+    "SHA256H q12, q13, v0.4s   \n"
+    "SHA256H2 q13, q11, v0.4s  \n"
+
+    /* Round 4 */
+    "SHA256SU0 v2.4s, v3.4s        \n"
+    "ADD v0.4s, v3.4s, v19.4s      \n"
+    "MOV v11.16b, v12.16b          \n"
+    "SHA256SU1 v2.4s, v4.4s, v1.4s \n"
+    "SHA256H q12, q13, v0.4s       \n"
+    "SHA256H2 q13, q11, v0.4s      \n"
+
+    /* Round 5 */
+    "SHA256SU0 v3.4s, v4.4s        \n"
+    "ADD v0.4s, v4.4s, v20.4s      \n"
+    "MOV v11.16b, v12.16b          \n"
+    "SHA256SU1 v3.4s, v1.4s, v2.4s \n"
+    "SHA256H q12, q13, v0.4s       \n"
+    "SHA256H2 q13, q11, v0.4s      \n"
+
+    /* Round 6 */
+    "SHA256SU0 v4.4s, v1.4s        \n"
+    "ADD v0.4s, v1.4s, v21.4s      \n"
+    "MOV v11.16b, v12.16b          \n"
+    "SHA256SU1 v4.4s, v2.4s, v3.4s \n"
+    "SHA256H q12, q13, v0.4s       \n"
+    "SHA256H2 q13, q11, v0.4s      \n"
+
+    /* Round 7 */
+    "SHA256SU0 v1.4s, v2.4s        \n"
+    "ADD v0.4s, v2.4s, v22.4s      \n"
+    "MOV v11.16b, v12.16b          \n"
+    "SHA256SU1 v1.4s, v3.4s, v4.4s \n"
+    "SHA256H q12, q13, v0.4s       \n"
+    "SHA256H2 q13, q11, v0.4s      \n"
+
+    /* Round 8 */
+    "SHA256SU0 v2.4s, v3.4s        \n"
+    "ADD v0.4s, v3.4s, v23.4s      \n"
+    "MOV v11.16b, v12.16b          \n"
+    "SHA256SU1 v2.4s, v4.4s, v1.4s \n"
+    "SHA256H q12, q13, v0.4s       \n"
+    "SHA256H2 q13, q11, v0.4s      \n"
+
+    /* Round 9 */
+    "SHA256SU0 v3.4s, v4.4s        \n"
+    "ADD v0.4s, v4.4s, v24.4s      \n"
+    "MOV v11.16b, v12.16b          \n"
+    "SHA256SU1 v3.4s, v1.4s, v2.4s \n"
+    "SHA256H q12, q13, v0.4s       \n"
+    "SHA256H2 q13, q11, v0.4s      \n"
+
+    /* Round 10 */
+    "SHA256SU0 v4.4s, v1.4s        \n"
+    "ADD v0.4s, v1.4s, v25.4s      \n"
+    "MOV v11.16b, v12.16b          \n"
+    "SHA256SU1 v4.4s, v2.4s, v3.4s \n"
+    "SHA256H q12, q13, v0.4s       \n"
+    "SHA256H2 q13, q11, v0.4s      \n"
+
+    /* Round 11 */
+    "SHA256SU0 v1.4s, v2.4s        \n"
+    "ADD v0.4s, v2.4s, v26.4s      \n"
+    "MOV v11.16b, v12.16b          \n"
+    "SHA256SU1 v1.4s, v3.4s, v4.4s \n"
+    "SHA256H q12, q13, v0.4s       \n"
+    "SHA256H2 q13, q11, v0.4s      \n"
+
+    /* Round 12 */
+    "SHA256SU0 v2.4s, v3.4s        \n"
+    "ADD v0.4s, v3.4s, v27.4s      \n"
+    "MOV v11.16b, v12.16b          \n"
+    "SHA256SU1 v2.4s, v4.4s, v1.4s \n"
+    "SHA256H q12, q13, v0.4s       \n"
+    "SHA256H2 q13, q11, v0.4s      \n"
+
+    /* Round 13 */
+    "SHA256SU0 v3.4s, v4.4s        \n"
+    "ADD v0.4s, v4.4s, v28.4s      \n"
+    "MOV v11.16b, v12.16b          \n"
+    "SHA256SU1 v3.4s, v1.4s, v2.4s \n"
+    "SHA256H q12, q13, v0.4s       \n"
+    "SHA256H2 q13, q11, v0.4s      \n"
+
+    /* Round 14 */
+    "ADD v0.4s, v1.4s, v29.4s      \n"
+    "MOV v11.16b, v12.16b          \n"
+    "SHA256H q12, q13, v0.4s       \n"
+    "SHA256H2 q13, q11, v0.4s      \n"
+
+    /* Round 15 */
+    "ADD v0.4s, v2.4s, v30.4s      \n"
+    "MOV v11.16b, v12.16b          \n"
+    "SHA256H q12, q13, v0.4s       \n"
+    "SHA256H2 q13, q11, v0.4s      \n"
+
+    /* Round 16 */
+    "ADD v0.4s, v3.4s, v31.4s      \n"
+    "MOV v11.16b, v12.16b          \n"
+    "SHA256H q12, q13, v0.4s       \n"
+    "SHA256H2 q13, q11, v0.4s      \n"
+
+    "#Add working vars back into digest state \n"
+    "SUB w8, w8, #1    \n"
+    "ADD v12.4s, v12.4s, v14.4s \n"
+    "ADD v13.4s, v13.4s, v15.4s \n"
+
+    "#check if more blocks should be done\n"
+    "CBZ w8, 2f \n"
+
+    "#load in message and schedule updates \n"
+    "LD1 {v0.2d-v3.2d}, [%[dataIn]], #64   \n"
+    "MOV v14.16b, v12.16b \n"
+    "MOV v15.16b, v13.16b \n"
+    "REV32 v0.16b, v0.16b \n"
+    "REV32 v1.16b, v1.16b \n"
+    "REV32 v2.16b, v2.16b \n"
+    "REV32 v3.16b, v3.16b \n"
+    "B 1b \n" /* do another block */
+
+    "2:\n"
+    "STP q12, q13, %[out] \n"
+
+    : [out] "=m" (sha256->digest), "=m" (sha256->buffer), "=r" (numBlocks),
+      "=r" (data), "=r" (k)
+    : [k] "4" (k), [digest] "m" (sha256->digest), [buffer] "m" (sha256->buffer),
+      [blocks] "2" (numBlocks), [dataIn] "3" (data)
+    : "cc", "memory", "v0", "v1", "v2", "v3", "v4", "v5", "v6", "v7",
+                      "v8",  "v9",  "v10", "v11", "v12", "v13", "v14",
+                      "v15", "v16", "v17", "v18", "v19", "v20", "v21",
+                      "v22", "v23", "v24", "v25", "v26", "v27", "v28",
+                      "v29", "v30", "v31", "w8"
+    );
+}
+
 /* ARMv8 hardware acceleration */
 static WC_INLINE int Sha256Update(wc_Sha256* sha256, const byte* data, word32 len)
 {
@@ -115,184 +324,11 @@ static WC_INLINE int Sha256Update(wc_Sha256* sha256, const byte* data, word32 le
         numBlocks = (len + sha256->buffLen)/WC_SHA256_BLOCK_SIZE;
 
         if (numBlocks > 0) {
-            word32* k = (word32*)K;
-
             /* get leftover amount after blocks */
             add = (len + sha256->buffLen) - numBlocks * WC_SHA256_BLOCK_SIZE;
-            __asm__ volatile (
-            "#load leftover data\n"
-            "LD1 {v0.2d-v3.2d}, %[buffer]   \n"
 
-            "#load current digest\n"
-            "LD1 {v12.2d-v13.2d}, %[digest] \n"
-            "MOV w8, %w[blocks] \n"
-            "REV32 v0.16b, v0.16b \n"
-            "REV32 v1.16b, v1.16b \n"
-            "REV32 v2.16b, v2.16b \n"
-            "REV32 v3.16b, v3.16b \n"
-
-            "#load K values in \n"
-            "LD1 {v16.4s-v19.4s}, [%[k]], #64    \n"
-            "LD1 {v20.4s-v23.4s}, [%[k]], #64    \n"
-            "MOV v14.16b, v12.16b \n" /* store digest for add at the end */
-            "MOV v15.16b, v13.16b \n"
-            "LD1 {v24.4s-v27.4s}, [%[k]], #64    \n"
-            "LD1 {v28.4s-v31.4s}, [%[k]], #64    \n"
-
-            /* beginning of SHA256 block operation */
-            "1:\n"
-            /* Round 1 */
-            "MOV v4.16b, v0.16b        \n"
-            "ADD v0.4s, v0.4s, v16.4s  \n"
-            "MOV v11.16b, v12.16b      \n"
-            "SHA256H q12, q13, v0.4s   \n"
-            "SHA256H2 q13, q11, v0.4s  \n"
-
-            /* Round 2 */
-            "SHA256SU0 v4.4s, v1.4s        \n"
-            "ADD v0.4s, v1.4s, v17.4s  \n"
-            "MOV v11.16b, v12.16b      \n"
-            "SHA256SU1 v4.4s, v2.4s, v3.4s \n"
-            "SHA256H q12, q13, v0.4s   \n"
-            "SHA256H2 q13, q11, v0.4s  \n"
-
-            /* Round 3 */
-            "SHA256SU0 v1.4s, v2.4s        \n"
-            "ADD v0.4s, v2.4s, v18.4s  \n"
-            "MOV v11.16b, v12.16b      \n"
-            "SHA256SU1 v1.4s, v3.4s, v4.4s \n"
-            "SHA256H q12, q13, v0.4s   \n"
-            "SHA256H2 q13, q11, v0.4s  \n"
-
-            /* Round 4 */
-            "SHA256SU0 v2.4s, v3.4s        \n"
-            "ADD v0.4s, v3.4s, v19.4s      \n"
-            "MOV v11.16b, v12.16b          \n"
-            "SHA256SU1 v2.4s, v4.4s, v1.4s \n"
-            "SHA256H q12, q13, v0.4s       \n"
-            "SHA256H2 q13, q11, v0.4s      \n"
-
-            /* Round 5 */
-            "SHA256SU0 v3.4s, v4.4s        \n"
-            "ADD v0.4s, v4.4s, v20.4s      \n"
-            "MOV v11.16b, v12.16b          \n"
-            "SHA256SU1 v3.4s, v1.4s, v2.4s \n"
-            "SHA256H q12, q13, v0.4s       \n"
-            "SHA256H2 q13, q11, v0.4s      \n"
-
-            /* Round 6 */
-            "SHA256SU0 v4.4s, v1.4s        \n"
-            "ADD v0.4s, v1.4s, v21.4s      \n"
-            "MOV v11.16b, v12.16b          \n"
-            "SHA256SU1 v4.4s, v2.4s, v3.4s \n"
-            "SHA256H q12, q13, v0.4s       \n"
-            "SHA256H2 q13, q11, v0.4s      \n"
-
-            /* Round 7 */
-            "SHA256SU0 v1.4s, v2.4s        \n"
-            "ADD v0.4s, v2.4s, v22.4s      \n"
-            "MOV v11.16b, v12.16b          \n"
-            "SHA256SU1 v1.4s, v3.4s, v4.4s \n"
-            "SHA256H q12, q13, v0.4s       \n"
-            "SHA256H2 q13, q11, v0.4s      \n"
-
-            /* Round 8 */
-            "SHA256SU0 v2.4s, v3.4s        \n"
-            "ADD v0.4s, v3.4s, v23.4s      \n"
-            "MOV v11.16b, v12.16b          \n"
-            "SHA256SU1 v2.4s, v4.4s, v1.4s \n"
-            "SHA256H q12, q13, v0.4s       \n"
-            "SHA256H2 q13, q11, v0.4s      \n"
-
-            /* Round 9 */
-            "SHA256SU0 v3.4s, v4.4s        \n"
-            "ADD v0.4s, v4.4s, v24.4s      \n"
-            "MOV v11.16b, v12.16b          \n"
-            "SHA256SU1 v3.4s, v1.4s, v2.4s \n"
-            "SHA256H q12, q13, v0.4s       \n"
-            "SHA256H2 q13, q11, v0.4s      \n"
-
-            /* Round 10 */
-            "SHA256SU0 v4.4s, v1.4s        \n"
-            "ADD v0.4s, v1.4s, v25.4s      \n"
-            "MOV v11.16b, v12.16b          \n"
-            "SHA256SU1 v4.4s, v2.4s, v3.4s \n"
-            "SHA256H q12, q13, v0.4s       \n"
-            "SHA256H2 q13, q11, v0.4s      \n"
-
-            /* Round 11 */
-            "SHA256SU0 v1.4s, v2.4s        \n"
-            "ADD v0.4s, v2.4s, v26.4s      \n"
-            "MOV v11.16b, v12.16b          \n"
-            "SHA256SU1 v1.4s, v3.4s, v4.4s \n"
-            "SHA256H q12, q13, v0.4s       \n"
-            "SHA256H2 q13, q11, v0.4s      \n"
-
-            /* Round 12 */
-            "SHA256SU0 v2.4s, v3.4s        \n"
-            "ADD v0.4s, v3.4s, v27.4s      \n"
-            "MOV v11.16b, v12.16b          \n"
-            "SHA256SU1 v2.4s, v4.4s, v1.4s \n"
-            "SHA256H q12, q13, v0.4s       \n"
-            "SHA256H2 q13, q11, v0.4s      \n"
-
-            /* Round 13 */
-            "SHA256SU0 v3.4s, v4.4s        \n"
-            "ADD v0.4s, v4.4s, v28.4s      \n"
-            "MOV v11.16b, v12.16b          \n"
-            "SHA256SU1 v3.4s, v1.4s, v2.4s \n"
-            "SHA256H q12, q13, v0.4s       \n"
-            "SHA256H2 q13, q11, v0.4s      \n"
-
-            /* Round 14 */
-            "ADD v0.4s, v1.4s, v29.4s      \n"
-            "MOV v11.16b, v12.16b          \n"
-            "SHA256H q12, q13, v0.4s       \n"
-            "SHA256H2 q13, q11, v0.4s      \n"
-
-            /* Round 15 */
-            "ADD v0.4s, v2.4s, v30.4s      \n"
-            "MOV v11.16b, v12.16b          \n"
-            "SHA256H q12, q13, v0.4s       \n"
-            "SHA256H2 q13, q11, v0.4s      \n"
-
-            /* Round 16 */
-            "ADD v0.4s, v3.4s, v31.4s      \n"
-            "MOV v11.16b, v12.16b          \n"
-            "SHA256H q12, q13, v0.4s       \n"
-            "SHA256H2 q13, q11, v0.4s      \n"
-
-            "#Add working vars back into digest state \n"
-            "SUB w8, w8, #1    \n"
-            "ADD v12.4s, v12.4s, v14.4s \n"
-            "ADD v13.4s, v13.4s, v15.4s \n"
-
-            "#check if more blocks should be done\n"
-            "CBZ w8, 2f \n"
-
-            "#load in message and schedule updates \n"
-            "LD1 {v0.2d-v3.2d}, [%[dataIn]], #64   \n"
-            "MOV v14.16b, v12.16b \n"
-            "MOV v15.16b, v13.16b \n"
-            "REV32 v0.16b, v0.16b \n"
-            "REV32 v1.16b, v1.16b \n"
-            "REV32 v2.16b, v2.16b \n"
-            "REV32 v3.16b, v3.16b \n"
-            "B 1b \n" /* do another block */
-
-            "2:\n"
-            "STP q12, q13, %[out] \n"
-
-            : [out] "=m" (sha256->digest), "=m" (sha256->buffer), "=r" (numBlocks),
-              "=r" (data), "=r" (k)
-            : [k] "4" (k), [digest] "m" (sha256->digest), [buffer] "m" (sha256->buffer),
-              [blocks] "2" (numBlocks), [dataIn] "3" (data)
-            : "cc", "memory", "v0", "v1", "v2", "v3", "v4", "v5", "v6", "v7",
-                              "v8",  "v9",  "v10", "v11", "v12", "v13", "v14",
-                              "v15", "v16", "v17", "v18", "v19", "v20", "v21",
-                              "v22", "v23", "v24", "v25", "v26", "v27", "v28",
-                              "v29", "v30", "v31", "w8"
-            );
+            Sha256Transform(sha256, data, numBlocks);
+            data += numBlocks * WC_SHA256_BLOCK_SIZE - sha256->buffLen;
 
             AddLength(sha256, WC_SHA256_BLOCK_SIZE * numBlocks);
 
@@ -313,6 +349,7 @@ static WC_INLINE int Sha256Update(wc_Sha256* sha256, const byte* data, word32 le
 static WC_INLINE int Sha256Final(wc_Sha256* sha256, byte* hash)
 {
     byte* local;
+    const word32* k;
 
     local = (byte*)sha256->buffer;
     AddLength(sha256, sha256->buffLen);  /* before adding pads */
@@ -324,6 +361,7 @@ static WC_INLINE int Sha256Final(wc_Sha256* sha256, byte* hash)
 
         XMEMSET(&local[sha256->buffLen], 0, WC_SHA256_BLOCK_SIZE - sha256->buffLen);
         sha256->buffLen += WC_SHA256_BLOCK_SIZE - sha256->buffLen;
+        k = K;
         __asm__ volatile (
             "LD1 {v4.2d-v7.2d}, %[buffer]          \n"
             "MOV v0.16b, v4.16b                    \n"
@@ -465,8 +503,8 @@ static WC_INLINE int Sha256Final(wc_Sha256* sha256, byte* hash)
             "ADD v17.4s, v17.4s, v21.4s \n"
             "STP q16, q17, %[out] \n"
 
-            : [out] "=m" (sha256->digest)
-            : [k] "r" (K), [digest] "m" (sha256->digest),
+            : [out] "=m" (sha256->digest), [k] "+r" (k)
+            : [digest] "m" (sha256->digest),
               [buffer] "m" (sha256->buffer)
             : "cc", "memory", "v0", "v1", "v2", "v3", "v8",  "v9",  "v10", "v11"
                             , "v12", "v13", "v14", "v15", "v16", "v17", "v18"
@@ -501,6 +539,7 @@ static WC_INLINE int Sha256Final(wc_Sha256* sha256, byte* hash)
     XMEMCPY(&local[WC_SHA256_PAD_SIZE + sizeof(word32)], &sha256->loLen,
             sizeof(word32));
 
+    k = K;
     __asm__ volatile (
         "#load in message and schedule updates \n"
         "LD1 {v4.2d-v7.2d}, %[buffer]        \n"
@@ -643,8 +682,8 @@ static WC_INLINE int Sha256Final(wc_Sha256* sha256, byte* hash)
         "REV32 v17.16b, v17.16b \n"
     #endif
         "ST1 {v17.16b}, [%[hashOut]] \n"
-        : [hashOut] "=r" (hash)
-        : [k] "r" (K), [digest] "m" (sha256->digest),
+        : [hashOut] "=r" (hash), [k] "+r" (k)
+        : [digest] "m" (sha256->digest),
           [buffer] "m" (sha256->buffer),
           "0" (hash)
             : "cc", "memory", "v0", "v1", "v2", "v3", "v4", "v5", "v6", "v7",
@@ -658,6 +697,202 @@ static WC_INLINE int Sha256Final(wc_Sha256* sha256, byte* hash)
 
 #else /* not using 64 bit */
 
+static WC_INLINE void Sha256Transform(wc_Sha256* sha256, const byte* data,
+                                      word32 numBlocks)
+{
+    word32* bufPt = sha256->buffer;
+    word32* digPt = sha256->digest;
+
+    __asm__ volatile (
+    "#load leftover data\n"
+    "VLDM %[buffer]!, {q0-q3} \n"
+
+    "#load current digest\n"
+    "VLDM %[digest], {q12-q13} \n"
+    "MOV r8, %[blocks] \n"
+    "VREV32.8 q0, q0 \n"
+    "VREV32.8 q1, q1 \n"
+    "VREV32.8 q2, q2 \n"
+    "VREV32.8 q3, q3 \n"
+    "VLDM %[k]! ,{q5-q8} \n"
+    "VLDM %[k]! ,{q9}\n"
+
+    "VMOV.32 q14, q12 \n" /* store digest for add at the end */
+    "VMOV.32 q15, q13 \n"
+
+    /* beginning of SHA256 block operation */
+    "1:\n"
+
+    /* Round 1 */
+    "VMOV.32 q4, q0           \n"
+    "VADD.i32 q0, q0, q5      \n"
+    "VMOV.32 q11, q12         \n"
+    "SHA256H.32 q12, q13, q0  \n"
+    "SHA256H2.32 q13, q11, q0 \n"
+
+    /* Round 2 */
+    "SHA256SU0.32 q4, q1      \n"
+    "VADD.i32 q0, q1, q6      \n"
+    "VMOV.32 q11, q12         \n"
+    "SHA256SU1.32 q4, q2, q3  \n"
+    "SHA256H.32 q12, q13, q0  \n"
+    "SHA256H2.32 q13, q11, q0 \n"
+
+    /* Round 3 */
+    "SHA256SU0.32 q1, q2      \n"
+    "VADD.i32 q0, q2, q7      \n"
+    "VMOV.32 q11, q12         \n"
+    "SHA256SU1.32 q1, q3, q4  \n"
+    "SHA256H.32 q12, q13, q0  \n"
+    "SHA256H2.32 q13, q11, q0 \n"
+
+    /* Round 4 */
+    "SHA256SU0.32 q2, q3      \n"
+    "VADD.i32 q0, q3, q8      \n"
+    "VMOV.32 q11, q12         \n"
+    "SHA256SU1.32 q2, q4, q1  \n"
+    "SHA256H.32 q12, q13, q0  \n"
+    "SHA256H2.32 q13, q11, q0 \n"
+
+    /* Round 5 */
+    "SHA256SU0.32 q3, q4      \n"
+    "VADD.i32 q0, q4, q9      \n"
+    "VMOV.32 q11, q12         \n"
+    "SHA256SU1.32 q3, q1, q2  \n"
+    "SHA256H.32 q12, q13, q0  \n"
+    "SHA256H2.32 q13, q11, q0 \n"
+
+    /* Round 6 */
+    "VLD1.32 {q10}, [%[k]]!   \n"
+    "SHA256SU0.32 q4, q1      \n"
+    "VADD.i32 q0, q1, q10     \n"
+    "VMOV.32 q11, q12         \n"
+    "SHA256SU1.32 q4, q2, q3  \n"
+    "SHA256H.32 q12, q13, q0  \n"
+    "SHA256H2.32 q13, q11, q0 \n"
+
+    /* Round 7 */
+    "VLD1.32 {q10}, [%[k]]!   \n"
+    "SHA256SU0.32 q1, q2      \n"
+    "VADD.i32 q0, q2, q10     \n"
+    "VMOV.32 q11, q12         \n"
+    "SHA256SU1.32 q1, q3, q4  \n"
+    "SHA256H.32 q12, q13, q0  \n"
+    "SHA256H2.32 q13, q11, q0 \n"
+
+    /* Round 8 */
+    "VLD1.32 {q10}, [%[k]]!   \n"
+    "SHA256SU0.32 q2, q3      \n"
+    "VADD.i32 q0, q3, q10     \n"
+    "VMOV.32 q11, q12         \n"
+    "SHA256SU1.32 q2, q4, q1  \n"
+    "SHA256H.32 q12, q13, q0  \n"
+    "SHA256H2.32 q13, q11, q0 \n"
+
+    /* Round 9 */
+    "VLD1.32 {q10}, [%[k]]!   \n"
+    "SHA256SU0.32 q3, q4      \n"
+    "VADD.i32 q0, q4, q10     \n"
+    "VMOV.32 q11, q12         \n"
+    "SHA256SU1.32 q3, q1, q2  \n"
+    "SHA256H.32 q12, q13, q0  \n"
+    "SHA256H2.32 q13, q11, q0 \n"
+
+    /* Round 10 */
+    "VLD1.32 {q10}, [%[k]]!   \n"
+    "SHA256SU0.32 q4, q1      \n"
+    "VADD.i32 q0, q1, q10     \n"
+    "VMOV.32 q11, q12         \n"
+    "SHA256SU1.32 q4, q2, q3  \n"
+    "SHA256H.32 q12, q13, q0  \n"
+    "SHA256H2.32 q13, q11, q0 \n"
+
+    /* Round 11 */
+    "VLD1.32 {q10}, [%[k]]!   \n"
+    "SHA256SU0.32 q1, q2      \n"
+    "VADD.i32 q0, q2, q10     \n"
+    "VMOV.32 q11, q12         \n"
+    "SHA256SU1.32 q1, q3, q4  \n"
+    "SHA256H.32 q12, q13, q0  \n"
+    "SHA256H2.32 q13, q11, q0 \n"
+
+    /* Round 12 */
+    "VLD1.32 {q10}, [%[k]]!   \n"
+    "SHA256SU0.32 q2, q3      \n"
+    "VADD.i32 q0, q3, q10     \n"
+    "VMOV.32 q11, q12         \n"
+    "SHA256SU1.32 q2, q4, q1  \n"
+    "SHA256H.32 q12, q13, q0  \n"
+    "SHA256H2.32 q13, q11, q0 \n"
+
+    /* Round 13 */
+    "VLD1.32 {q10}, [%[k]]!   \n"
+    "SHA256SU0.32 q3, q4      \n"
+    "VADD.i32 q0, q4, q10     \n"
+    "VMOV.32 q11, q12         \n"
+    "SHA256SU1.32 q3, q1, q2  \n"
+    "SHA256H.32 q12, q13, q0  \n"
+    "SHA256H2.32 q13, q11, q0 \n"
+
+    /* Round 14 */
+    "VLD1.32 {q10}, [%[k]]!   \n"
+    "VADD.i32 q0, q1, q10     \n"
+    "VMOV.32 q11, q12         \n"
+    "SHA256H.32 q12, q13, q0  \n"
+    "SHA256H2.32 q13, q11, q0 \n"
+
+    /* Round 15 */
+    "VLD1.32 {q10}, [%[k]]!   \n"
+    "VADD.i32 q0, q2, q10     \n"
+    "VMOV.32 q11, q12         \n"
+    "SHA256H.32 q12, q13, q0  \n"
+    "SHA256H2.32 q13, q11, q0 \n"
+
+    /* Round 16 */
+    "VLD1.32 {q10}, [%[k]]    \n"
+    "SUB r8, r8, #1           \n"
+    "VADD.i32 q0, q3, q10     \n"
+    "VMOV.32 q11, q12         \n"
+    "SHA256H.32 q12, q13, q0  \n"
+    "SHA256H2.32 q13, q11, q0 \n"
+
+    "#Add working vars back into digest state \n"
+    "VADD.i32 q12, q12, q14 \n"
+    "VADD.i32 q13, q13, q15 \n"
+
+    "#check if more blocks should be done\n"
+    "CMP r8, #0 \n"
+    "BEQ 2f \n"
+
+    "#load in message and schedule updates \n"
+    "VLD1.32 {q0}, [%[dataIn]]!   \n"
+    "VLD1.32 {q1}, [%[dataIn]]!   \n"
+    "VLD1.32 {q2}, [%[dataIn]]!   \n"
+    "VLD1.32 {q3}, [%[dataIn]]!   \n"
+
+    /* reset K pointer */
+    "SUB %[k], %[k], #160 \n"
+    "VREV32.8 q0, q0 \n"
+    "VREV32.8 q1, q1 \n"
+    "VREV32.8 q2, q2 \n"
+    "VREV32.8 q3, q3 \n"
+    "VMOV.32 q14, q12 \n"
+    "VMOV.32 q15, q13 \n"
+    "B 1b \n" /* do another block */
+
+    "2:\n"
+    "VST1.32 {q12, q13}, [%[out]] \n"
+
+    : [out] "=r" (digPt), "=r" (bufPt), "=r" (numBlocks),
+      "=r" (data)
+    : [k] "r" (K), [digest] "0" (digPt), [buffer] "1" (bufPt),
+      [blocks] "2" (numBlocks), [dataIn] "3" (data)
+    : "cc", "memory", "q0", "q1", "q2", "q3", "q4", "q5", "q6", "q7",
+                      "q8",  "q9",  "q10", "q11", "q12", "q13", "q14",
+                      "q15", "r8"
+    );
+}
+
 /* ARMv8 hardware acceleration Aarch32 */
 static WC_INLINE int Sha256Update(wc_Sha256* sha256, const byte* data, word32 len)
 {
@@ -677,198 +912,11 @@ static WC_INLINE int Sha256Update(wc_Sha256* sha256, const byte* data, word32 le
         numBlocks = (len + sha256->buffLen)/WC_SHA256_BLOCK_SIZE;
 
         if (numBlocks > 0) {
-            word32* bufPt = sha256->buffer;
-            word32* digPt = sha256->digest;
             /* get leftover amount after blocks */
             add = (len + sha256->buffLen) - numBlocks * WC_SHA256_BLOCK_SIZE;
-            __asm__ volatile (
-            "#load leftover data\n"
-            "VLDM %[buffer]!, {q0-q3} \n"
 
-            "#load current digest\n"
-            "VLDM %[digest], {q12-q13} \n"
-            "MOV r8, %[blocks] \n"
-            "VREV32.8 q0, q0 \n"
-            "VREV32.8 q1, q1 \n"
-            "VREV32.8 q2, q2 \n"
-            "VREV32.8 q3, q3 \n"
-            "VLDM %[k]! ,{q5-q8} \n"
-            "VLDM %[k]! ,{q9}\n"
-
-            "VMOV.32 q14, q12 \n" /* store digest for add at the end */
-            "VMOV.32 q15, q13 \n"
-
-            /* beginning of SHA256 block operation */
-            "1:\n"
-
-            /* Round 1 */
-            "VMOV.32 q4, q0           \n"
-            "VADD.i32 q0, q0, q5      \n"
-            "VMOV.32 q11, q12         \n"
-            "SHA256H.32 q12, q13, q0  \n"
-            "SHA256H2.32 q13, q11, q0 \n"
-
-            /* Round 2 */
-            "SHA256SU0.32 q4, q1      \n"
-            "VADD.i32 q0, q1, q6      \n"
-            "VMOV.32 q11, q12         \n"
-            "SHA256SU1.32 q4, q2, q3  \n"
-            "SHA256H.32 q12, q13, q0  \n"
-            "SHA256H2.32 q13, q11, q0 \n"
-
-            /* Round 3 */
-            "SHA256SU0.32 q1, q2      \n"
-            "VADD.i32 q0, q2, q7      \n"
-            "VMOV.32 q11, q12         \n"
-            "SHA256SU1.32 q1, q3, q4  \n"
-            "SHA256H.32 q12, q13, q0  \n"
-            "SHA256H2.32 q13, q11, q0 \n"
-
-            /* Round 4 */
-            "SHA256SU0.32 q2, q3      \n"
-            "VADD.i32 q0, q3, q8      \n"
-            "VMOV.32 q11, q12         \n"
-            "SHA256SU1.32 q2, q4, q1  \n"
-            "SHA256H.32 q12, q13, q0  \n"
-            "SHA256H2.32 q13, q11, q0 \n"
-
-            /* Round 5 */
-            "SHA256SU0.32 q3, q4      \n"
-            "VADD.i32 q0, q4, q9      \n"
-            "VMOV.32 q11, q12         \n"
-            "SHA256SU1.32 q3, q1, q2  \n"
-            "SHA256H.32 q12, q13, q0  \n"
-            "SHA256H2.32 q13, q11, q0 \n"
-
-            /* Round 6 */
-            "VLD1.32 {q10}, [%[k]]!   \n"
-            "SHA256SU0.32 q4, q1      \n"
-            "VADD.i32 q0, q1, q10     \n"
-            "VMOV.32 q11, q12         \n"
-            "SHA256SU1.32 q4, q2, q3  \n"
-            "SHA256H.32 q12, q13, q0  \n"
-            "SHA256H2.32 q13, q11, q0 \n"
-
-            /* Round 7 */
-            "VLD1.32 {q10}, [%[k]]!   \n"
-            "SHA256SU0.32 q1, q2      \n"
-            "VADD.i32 q0, q2, q10     \n"
-            "VMOV.32 q11, q12         \n"
-            "SHA256SU1.32 q1, q3, q4  \n"
-            "SHA256H.32 q12, q13, q0  \n"
-            "SHA256H2.32 q13, q11, q0 \n"
-
-            /* Round 8 */
-            "VLD1.32 {q10}, [%[k]]!   \n"
-            "SHA256SU0.32 q2, q3      \n"
-            "VADD.i32 q0, q3, q10     \n"
-            "VMOV.32 q11, q12         \n"
-            "SHA256SU1.32 q2, q4, q1  \n"
-            "SHA256H.32 q12, q13, q0  \n"
-            "SHA256H2.32 q13, q11, q0 \n"
-
-            /* Round 9 */
-            "VLD1.32 {q10}, [%[k]]!   \n"
-            "SHA256SU0.32 q3, q4      \n"
-            "VADD.i32 q0, q4, q10     \n"
-            "VMOV.32 q11, q12         \n"
-            "SHA256SU1.32 q3, q1, q2  \n"
-            "SHA256H.32 q12, q13, q0  \n"
-            "SHA256H2.32 q13, q11, q0 \n"
-
-            /* Round 10 */
-            "VLD1.32 {q10}, [%[k]]!   \n"
-            "SHA256SU0.32 q4, q1      \n"
-            "VADD.i32 q0, q1, q10     \n"
-            "VMOV.32 q11, q12         \n"
-            "SHA256SU1.32 q4, q2, q3  \n"
-            "SHA256H.32 q12, q13, q0  \n"
-            "SHA256H2.32 q13, q11, q0 \n"
-
-            /* Round 11 */
-            "VLD1.32 {q10}, [%[k]]!   \n"
-            "SHA256SU0.32 q1, q2      \n"
-            "VADD.i32 q0, q2, q10     \n"
-            "VMOV.32 q11, q12         \n"
-            "SHA256SU1.32 q1, q3, q4  \n"
-            "SHA256H.32 q12, q13, q0  \n"
-            "SHA256H2.32 q13, q11, q0 \n"
-
-            /* Round 12 */
-            "VLD1.32 {q10}, [%[k]]!   \n"
-            "SHA256SU0.32 q2, q3      \n"
-            "VADD.i32 q0, q3, q10     \n"
-            "VMOV.32 q11, q12         \n"
-            "SHA256SU1.32 q2, q4, q1  \n"
-            "SHA256H.32 q12, q13, q0  \n"
-            "SHA256H2.32 q13, q11, q0 \n"
-
-            /* Round 13 */
-            "VLD1.32 {q10}, [%[k]]!   \n"
-            "SHA256SU0.32 q3, q4      \n"
-            "VADD.i32 q0, q4, q10     \n"
-            "VMOV.32 q11, q12         \n"
-            "SHA256SU1.32 q3, q1, q2  \n"
-            "SHA256H.32 q12, q13, q0  \n"
-            "SHA256H2.32 q13, q11, q0 \n"
-
-            /* Round 14 */
-            "VLD1.32 {q10}, [%[k]]!   \n"
-            "VADD.i32 q0, q1, q10     \n"
-            "VMOV.32 q11, q12         \n"
-            "SHA256H.32 q12, q13, q0  \n"
-            "SHA256H2.32 q13, q11, q0 \n"
-
-            /* Round 15 */
-            "VLD1.32 {q10}, [%[k]]!   \n"
-            "VADD.i32 q0, q2, q10     \n"
-            "VMOV.32 q11, q12         \n"
-            "SHA256H.32 q12, q13, q0  \n"
-            "SHA256H2.32 q13, q11, q0 \n"
-
-            /* Round 16 */
-            "VLD1.32 {q10}, [%[k]]    \n"
-            "SUB r8, r8, #1           \n"
-            "VADD.i32 q0, q3, q10     \n"
-            "VMOV.32 q11, q12         \n"
-            "SHA256H.32 q12, q13, q0  \n"
-            "SHA256H2.32 q13, q11, q0 \n"
-
-            "#Add working vars back into digest state \n"
-            "VADD.i32 q12, q12, q14 \n"
-            "VADD.i32 q13, q13, q15 \n"
-
-            "#check if more blocks should be done\n"
-            "CMP r8, #0 \n"
-            "BEQ 2f \n"
-
-            "#load in message and schedule updates \n"
-            "VLD1.32 {q0}, [%[dataIn]]!   \n"
-            "VLD1.32 {q1}, [%[dataIn]]!   \n"
-            "VLD1.32 {q2}, [%[dataIn]]!   \n"
-            "VLD1.32 {q3}, [%[dataIn]]!   \n"
-
-            /* reset K pointer */
-            "SUB %[k], %[k], #160 \n"
-            "VREV32.8 q0, q0 \n"
-            "VREV32.8 q1, q1 \n"
-            "VREV32.8 q2, q2 \n"
-            "VREV32.8 q3, q3 \n"
-            "VMOV.32 q14, q12 \n"
-            "VMOV.32 q15, q13 \n"
-            "B 1b \n" /* do another block */
-
-            "2:\n"
-            "VST1.32 {q12, q13}, [%[out]] \n"
-
-            : [out] "=r" (digPt), "=r" (bufPt), "=r" (numBlocks),
-              "=r" (data)
-            : [k] "r" (K), [digest] "0" (digPt), [buffer] "1" (bufPt),
-              [blocks] "2" (numBlocks), [dataIn] "3" (data)
-            : "cc", "memory", "q0", "q1", "q2", "q3", "q4", "q5", "q6", "q7",
-                              "q8",  "q9",  "q10", "q11", "q12", "q13", "q14",
-                              "q15", "r8"
-            );
+            Sha256Transform(sha256, data, numBlocks);
+            data += numBlocks * WC_SHA256_BLOCK_SIZE - sha256->buffLen;
 
             AddLength(sha256, WC_SHA256_BLOCK_SIZE * numBlocks);
 
@@ -1288,6 +1336,157 @@ static WC_INLINE int Sha256Final(wc_Sha256* sha256, byte* hash)
 
 #endif /* __aarch64__ */
 
+#else /* WOLFSSL_ARMASM_NO_HW_CRYPTO */
+
+#if defined(FREESCALE_MMCAU_SHA)
+
+    #ifndef WC_HASH_DATA_ALIGNMENT
+        /* these hardware API's require 4 byte (word32) alignment */
+        #define WC_HASH_DATA_ALIGNMENT 4
+    #endif
+
+    static int Transform_Sha256_Len(wc_Sha256* sha256, const byte* data,
+        word32 len)
+    {
+        int ret = wolfSSL_CryptHwMutexLock();
+        if (ret == 0) {
+        #if defined(WC_HASH_DATA_ALIGNMENT) && WC_HASH_DATA_ALIGNMENT > 0
+            if ((wc_ptr_t)data % WC_HASH_DATA_ALIGNMENT) {
+                /* data pointer is NOT aligned,
+                 * so copy and perform one block at a time */
+                byte* local = (byte*)sha256->buffer;
+                while (len >= WC_SHA256_BLOCK_SIZE) {
+                    XMEMCPY(local, data, WC_SHA256_BLOCK_SIZE);
+                #ifdef FREESCALE_MMCAU_CLASSIC_SHA
+                    cau_sha256_hash_n(local, 1, sha256->digest);
+                #else
+                    MMCAU_SHA256_HashN(local, 1, (uint32_t*)sha256->digest);
+                #endif
+                    data += WC_SHA256_BLOCK_SIZE;
+                    len  -= WC_SHA256_BLOCK_SIZE;
+                }
+            }
+            else
+        #endif
+            {
+    #ifdef FREESCALE_MMCAU_CLASSIC_SHA
+            cau_sha256_hash_n((byte*)data, len/WC_SHA256_BLOCK_SIZE,
+                sha256->digest);
+    #else
+            MMCAU_SHA256_HashN((byte*)data, len/WC_SHA256_BLOCK_SIZE,
+                (uint32_t*)sha256->digest);
+    #endif
+            }
+            wolfSSL_CryptHwMutexUnLock();
+        }
+        return ret;
+    }
+
+#else /* */
+
+extern void Transform_Sha256_Len(wc_Sha256* sha256, const byte* data,
+    word32 len);
+
+#endif
+
+/* ARMv8 hardware acceleration Aarch32 and Thumb2 */
+static WC_INLINE int Sha256Update(wc_Sha256* sha256, const byte* data, word32 len)
+{
+    int ret = 0;
+    /* do block size increments */
+    byte* local = (byte*)sha256->buffer;
+    word32 blocksLen;
+
+    /* check that internal buffLen is valid */
+    if (sha256->buffLen >= WC_SHA256_BLOCK_SIZE)
+        return BUFFER_E;
+
+    AddLength(sha256, len);
+
+    if (sha256->buffLen > 0) {
+        word32 add = min(len, WC_SHA256_BLOCK_SIZE - sha256->buffLen);
+        if (add > 0) {
+            XMEMCPY(&local[sha256->buffLen], data, add);
+
+            sha256->buffLen += add;
+            data            += add;
+            len             -= add;
+        }
+
+        if (sha256->buffLen == WC_SHA256_BLOCK_SIZE) {
+            Transform_Sha256_Len(sha256, (const byte*)sha256->buffer,
+                                                          WC_SHA256_BLOCK_SIZE);
+            sha256->buffLen = 0;
+        }
+    }
+
+    blocksLen = len & ~(WC_SHA256_BLOCK_SIZE-1);
+    if (blocksLen > 0) {
+        /* Byte reversal performed in function if required. */
+        Transform_Sha256_Len(sha256, data, blocksLen);
+        data += blocksLen;
+        len  -= blocksLen;
+    }
+
+    if (len > 0) {
+        XMEMCPY(local, data, len);
+        sha256->buffLen = len;
+    }
+
+    return ret;
+}
+
+static WC_INLINE int Sha256Final(wc_Sha256* sha256, byte* hash)
+{
+    byte* local = (byte*)sha256->buffer;
+
+    if (sha256 == NULL) {
+        return BAD_FUNC_ARG;
+    }
+
+    local[sha256->buffLen++] = 0x80;  /* add 1 */
+
+    /* pad with zeros */
+    if (sha256->buffLen > WC_SHA256_PAD_SIZE) {
+        XMEMSET(&local[sha256->buffLen], 0, WC_SHA256_BLOCK_SIZE -
+                                                               sha256->buffLen);
+        sha256->buffLen += WC_SHA256_BLOCK_SIZE - sha256->buffLen;
+        Transform_Sha256_Len(sha256, (const byte*)sha256->buffer,
+                                                          WC_SHA256_BLOCK_SIZE);
+
+        sha256->buffLen = 0;
+    }
+    XMEMSET(&local[sha256->buffLen], 0, WC_SHA256_PAD_SIZE - sha256->buffLen);
+
+    /* put lengths in bits */
+    sha256->hiLen = (sha256->loLen >> (8 * sizeof(sha256->loLen) - 3)) +
+                                                         (sha256->hiLen << 3);
+    sha256->loLen = sha256->loLen << 3;
+
+    /* store lengths */
+    /* ! length ordering dependent on digest endian type ! */
+
+    sha256->buffer[WC_SHA256_BLOCK_SIZE / sizeof(word32) - 2] = sha256->hiLen;
+    sha256->buffer[WC_SHA256_BLOCK_SIZE / sizeof(word32) - 1] = sha256->loLen;
+
+    ByteReverseWords(
+                   &(sha256->buffer[WC_SHA256_BLOCK_SIZE / sizeof(word32) - 2]),
+                   &(sha256->buffer[WC_SHA256_BLOCK_SIZE / sizeof(word32) - 2]),
+                   WC_SHA256_BLOCK_SIZE - WC_SHA256_PAD_SIZE);
+    Transform_Sha256_Len(sha256, (const byte*)sha256->buffer,
+                                                          WC_SHA256_BLOCK_SIZE);
+
+#ifdef LITTLE_ENDIAN_ORDER
+    ByteReverseWords((word32*)hash, sha256->digest, WC_SHA256_DIGEST_SIZE);
+#else
+    XMEMCPY(hash, sha256->digest, WC_SHA256_DIGEST_SIZE);
+#endif
+
+    return 0;
+}
+
+#endif /* !WOLFSSL_ARMASM_NO_HW_CRYPTO */
+
 
 #ifndef NO_SHA256
 
@@ -1297,6 +1496,9 @@ int wc_InitSha256_ex(wc_Sha256* sha256, void* heap, int devId)
         return BAD_FUNC_ARG;
 
     sha256->heap = heap;
+#ifdef WOLF_CRYPTO_CB
+    sha256->devId = devId;
+#endif
     (void)devId;
 
     return InitSha256(sha256);
@@ -1372,7 +1574,7 @@ int wc_Sha256GetHash(wc_Sha256* sha256, byte* hash)
     return ret;
 }
 
-#if defined(WOLFSSL_HASH_FLAGS) || defined(WOLF_CRYPTO_CB)
+#ifdef WOLFSSL_HASH_FLAGS
 int wc_Sha256SetFlags(wc_Sha256* sha256, word32 flags)
 {
     if (sha256) {
@@ -1401,6 +1603,26 @@ int wc_Sha256Copy(wc_Sha256* src, wc_Sha256* dst)
     return ret;
 }
 
+#ifdef OPENSSL_EXTRA
+int wc_Sha256Transform(wc_Sha256* sha256, const unsigned char* data)
+{
+    if (sha256 == NULL || data == NULL) {
+        return BAD_FUNC_ARG;
+    }
+#ifdef LITTLE_ENDIAN_ORDER
+    ByteReverseWords(sha256->buffer, (word32*)data, WC_SHA256_BLOCK_SIZE);
+#else
+    XMEMCPY(sha256->buffer, data, WC_SHA256_BLOCK_SIZE);
+#endif
+#ifndef WOLFSSL_ARMASM_NO_HW_CRYPTO
+    Sha256Transform(sha256, data, 1);
+#else
+    Transform_Sha256_Len(sha256, data, WC_SHA256_BLOCK_SIZE);
+#endif
+    return 0;
+}
+#endif
+
 #endif /* !NO_SHA256 */
 
 
@@ -1427,6 +1649,9 @@ int wc_Sha256Copy(wc_Sha256* src, wc_Sha256* dst)
         sha224->loLen   = 0;
         sha224->hiLen   = 0;
 
+    #ifdef WOLFSSL_HASH_FLAGS
+        sha224->flags = 0;
+    #endif
         return ret;
     }
 
@@ -1498,7 +1723,7 @@ int wc_Sha256Copy(wc_Sha256* src, wc_Sha256* dst)
         return ret;
     }
 
-#if defined(WOLFSSL_HASH_FLAGS) || defined(WOLF_CRYPTO_CB)
+#ifdef WOLFSSL_HASH_FLAGS
     int wc_Sha224SetFlags(wc_Sha224* sha224, word32 flags)
     {
         if (sha224) {
