@@ -1,6 +1,6 @@
 /* sha512.c
  *
- * Copyright (C) 2006-2021 wolfSSL Inc.
+ * Copyright (C) 2006-2023 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
@@ -146,23 +146,6 @@ static int InitSha512_256(wc_Sha512* sha512)
 
 #ifdef WOLFSSL_SHA512
 
-#ifdef WOLFSSL_ARMASM
-#ifdef __aarch64__
-#ifndef WOLFSSL_ARMASM_CRYPTO_SHA512
-    extern void Transform_Sha512_Len_neon(wc_Sha512* sha512, const byte* data,
-        word32 len);
-    #define Transform_Sha512_Len    Transform_Sha512_Len_neon
-#else
-    extern void Transform_Sha512_Len_crypto(wc_Sha512* sha512, const byte* data,
-        word32 len);
-    #define Transform_Sha512_Len    Transform_Sha512_Len_crypto
-#endif
-#else
-extern void Transform_Sha512_Len(wc_Sha512* sha512, const byte* data,
-    word32 len);
-#endif
-#endif
-
 static int InitSha512_Family(wc_Sha512* sha512, void* heap, int devId,
                                                     enum wc_HashType type)
 {
@@ -472,7 +455,25 @@ static WC_INLINE int Sha512Update(wc_Sha512* sha512, const byte* data, word32 le
     blocksLen = len & ~(WC_SHA512_BLOCK_SIZE-1);
     if (blocksLen > 0) {
         /* Byte reversal performed in function if required. */
-        Transform_Sha512_Len(sha512, data, blocksLen);
+    #ifndef WOLFSSL_ARMASM_NO_NEON
+        /* Data must be 64-bit aligned to be passed to Transform_Sha512_Len().
+         * 64 bits is 8 bytes.
+         */
+        if (((size_t)data & 0x7) != 0) {
+            word32 i;
+
+            for (i = 0; i < blocksLen; i += WC_SHA512_BLOCK_SIZE) {
+                word64 buffer[WC_SHA512_BLOCK_SIZE / sizeof(word64)];
+                XMEMCPY(buffer, data + i, WC_SHA512_BLOCK_SIZE);
+                Transform_Sha512_Len(sha512, (const byte*)buffer,
+                                                          WC_SHA512_BLOCK_SIZE);
+            }
+        }
+        else
+    #endif
+        {
+            Transform_Sha512_Len(sha512, data, blocksLen);
+        }
         data += blocksLen;
         len  -= blocksLen;
     }
@@ -797,6 +798,8 @@ void wc_Sha384Free(wc_Sha384* sha384)
 
 #ifdef WOLFSSL_SHA512
 
+#if !defined(WOLFSSL_NOSHA512_224) || !defined(WOLFSSL_NOSHA512_256)
+
 static int Sha512_Family_GetHash(wc_Sha512* sha512, byte* hash,
                                             enum wc_HashType type )
 {
@@ -833,6 +836,8 @@ static int Sha512_Family_GetHash(wc_Sha512* sha512, byte* hash,
     return ret;
 }
 
+#endif /* !WOLFSSL_NOSHA512_224 || !WOLFSSL_NOSHA512_256 */
+
 int wc_Sha512GetHash(wc_Sha512* sha512, byte* hash)
 {
     int ret;
