@@ -1,6 +1,6 @@
 /* x509_vfy.h
  *
- * Copyright (C) 2006-2017 wolfSSL Inc.
+ * Copyright (C) 2006-2023 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
@@ -24,6 +24,7 @@
 #ifndef WOLFSSL_x509_vfy_H_
 #define WOLFSSL_x509_vfy_H_
 
+#include <wolfssl/openssl/compat_types.h>
 #include <wolfssl/openssl/x509v3.h>
 
 #ifdef __cplusplus
@@ -32,11 +33,12 @@
 
 #if defined(WOLFSSL_QT) || defined(OPENSSL_ALL)
     WOLFSSL_API int wolfSSL_X509_STORE_CTX_set_purpose(WOLFSSL_X509_STORE_CTX *ctx, int purpose);
+    WOLFSSL_API void wolfSSL_X509_STORE_CTX_set_flags(WOLFSSL_X509_STORE_CTX *ctx,
+        unsigned long flags);
 #endif
 
-#ifdef WOLFSSL_QT
-    #define X509_STORE_CTX_set_purpose  wolfSSL_X509_STORE_CTX_set_purpose
-#endif
+#define X509_STORE_CTX_set_purpose  wolfSSL_X509_STORE_CTX_set_purpose
+#define X509_STORE_CTX_set_flags    wolfSSL_X509_STORE_CTX_set_flags
 
 #ifdef  __cplusplus
 } /* extern "C" */
