@@ -1,6 +1,6 @@
 /* fe_operations.h
  *
- * Copyright (C) 2006-2020 wolfSSL Inc.
+ * Copyright (C) 2006-2023 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
@@ -27,12 +27,6 @@
 
 #if defined(HAVE_CURVE25519) || defined(HAVE_ED25519)
 
-#if !defined(CURVE25519_SMALL) || !defined(ED25519_SMALL)
-#ifndef WOLFSSL_LINUXKM
-    #include <stdint.h>
-#endif
-#endif
-
 #include <wolfssl/wolfcrypt/types.h>
 
 #if defined(USE_INTEL_SPEEDUP) && !defined(NO_CURVED25519_X64)
@@ -88,66 +82,44 @@ WOLFSSL_LOCAL int  curve25519(byte * q, const byte * n, const byte * p);
 #if !defined(CURVE25519_SMALL) || !defined(ED25519_SMALL)
 
 #ifdef CURVED25519_ASM_64BIT
-    typedef int64_t  fe[4];
+    typedef sword64  fe[4];
 #elif defined(CURVED25519_ASM_32BIT)
-    typedef int32_t  fe[8];
+    typedef sword32  fe[8];
 #elif defined(CURVED25519_128BIT)
-    typedef int64_t  fe[5];
+    typedef sword64  fe[5];
 #else
-    typedef int32_t  fe[10];
+    typedef sword32  fe[10];
 #endif
 
-WOLFSSL_LOCAL void fe_copy(fe, const fe);
-WOLFSSL_LOCAL void fe_add(fe, const fe, const fe);
-WOLFSSL_LOCAL void fe_neg(fe,const fe);
-WOLFSSL_LOCAL void fe_sub(fe, const fe, const fe);
-WOLFSSL_LOCAL void fe_invert(fe, const fe);
-WOLFSSL_LOCAL void fe_mul(fe,const fe,const fe);
+WOLFSSL_LOCAL void fe_copy(fe h,const fe f);
+WOLFSSL_LOCAL void fe_add(fe h,const fe f,const fe g);
+WOLFSSL_LOCAL void fe_neg(fe h,const fe f);
+WOLFSSL_LOCAL void fe_sub(fe h,const fe f,const fe g);
+WOLFSSL_LOCAL void fe_invert(fe out,const fe z);
+WOLFSSL_LOCAL void fe_mul(fe h,const fe f,const fe g);
 
 
 /* Based On Daniel J Bernstein's curve25519 and ed25519 Public Domain ref10
    work. */
 
-WOLFSSL_LOCAL void fe_0(fe);
-WOLFSSL_LOCAL void fe_1(fe);
-WOLFSSL_LOCAL int  fe_isnonzero(const fe);
-WOLFSSL_LOCAL int  fe_isnegative(const fe);
-WOLFSSL_LOCAL void fe_tobytes(unsigned char *, const fe);
-WOLFSSL_LOCAL void fe_sq(fe, const fe);
-WOLFSSL_LOCAL void fe_sq2(fe,const fe);
-WOLFSSL_LOCAL void fe_frombytes(fe,const unsigned char *);
-WOLFSSL_LOCAL void fe_cswap(fe, fe, int);
-WOLFSSL_LOCAL void fe_mul121666(fe,fe);
-WOLFSSL_LOCAL void fe_cmov(fe,const fe, int);
-WOLFSSL_LOCAL void fe_pow22523(fe,const fe);
+WOLFSSL_LOCAL void fe_0(fe h);
+WOLFSSL_LOCAL void fe_1(fe h);
+WOLFSSL_LOCAL int  fe_isnonzero(const fe f);
+WOLFSSL_LOCAL int  fe_isnegative(const fe f);
+WOLFSSL_LOCAL void fe_tobytes(unsigned char *s,const fe h);
+WOLFSSL_LOCAL void fe_sq(fe h,const fe f);
+WOLFSSL_LOCAL void fe_sq2(fe h,const fe f);
+WOLFSSL_LOCAL void fe_frombytes(fe h,const unsigned char *s);
+WOLFSSL_LOCAL void fe_cswap(fe f, fe g, int b);
+WOLFSSL_LOCAL void fe_mul121666(fe h,fe f);
+WOLFSSL_LOCAL void fe_cmov(fe f, const fe g, int b);
+WOLFSSL_LOCAL void fe_pow22523(fe out,const fe z);
 
 /* 64 type needed for SHA512 */
-WOLFSSL_LOCAL uint64_t load_3(const unsigned char *in);
-WOLFSSL_LOCAL uint64_t load_4(const unsigned char *in);
+WOLFSSL_LOCAL word64 load_3(const unsigned char *in);
+WOLFSSL_LOCAL word64 load_4(const unsigned char *in);
 
 #ifdef CURVED25519_ASM
-WOLFSSL_LOCAL void fe_ge_to_p2(fe rx, fe ry, fe rz, const fe px, const fe py,
-                               const fe pz, const fe pt);
-WOLFSSL_LOCAL void fe_ge_to_p3(fe rx, fe ry, fe rz, fe rt, const fe px,
-                               const fe py, const fe pz, const fe pt);
-WOLFSSL_LOCAL void fe_ge_dbl(fe rx, fe ry, fe rz, fe rt, const fe px,
-                             const fe py, const fe pz);
-WOLFSSL_LOCAL void fe_ge_madd(fe rx, fe ry, fe rz, fe rt, const fe px,
-                              const fe py, const fe pz, const fe pt,
-                              const fe qxy2d, const fe qyplusx,
-                              const fe qyminusx);
-WOLFSSL_LOCAL void fe_ge_msub(fe rx, fe ry, fe rz, fe rt, const fe px,
-                              const fe py, const fe pz, const fe pt,
-                              const fe qxy2d, const fe qyplusx,
-                              const fe qyminusx);
-WOLFSSL_LOCAL void fe_ge_add(fe rx, fe ry, fe rz, fe rt, const fe px,
-                             const fe py, const fe pz, const fe pt, const fe qz,
-                             const fe qt2d, const fe qyplusx,
-                             const fe qyminusx);
-WOLFSSL_LOCAL void fe_ge_sub(fe rx, fe ry, fe rz, fe rt, const fe px,
-                             const fe py, const fe pz, const fe pt, const fe qz,
-                             const fe qt2d, const fe qyplusx,
-                             const fe qyminusx);
 WOLFSSL_LOCAL void fe_cmov_table(fe* r, fe* base, signed char b);
 #endif /* CURVED25519_ASM */
 #endif /* !CURVE25519_SMALL || !ED25519_SMALL */
@@ -170,7 +142,7 @@ WOLFSSL_LOCAL void fe_inv__distinct(byte *r, const byte *x);
  * undefined behavior.
  */
 WOLFSSL_LOCAL void fe_select(byte *dst, const byte *zero, const byte *one,
-		   byte condition);
+                   byte condition);
 
 /* Multiply a point by a small constant. The two pointers are not
  * required to be distinct.
@@ -194,11 +166,11 @@ WOLFSSL_LOCAL void fe_sqrt(byte *r, const byte *x);
  * undefined behavior.
  */
 WOLFSSL_LOCAL void fprime_select(byte *dst, const byte *zero, const byte *one,
-		                         byte condition);
+                                         byte condition);
 WOLFSSL_LOCAL void fprime_add(byte *r, const byte *a, const byte *modulus);
 WOLFSSL_LOCAL void fprime_sub(byte *r, const byte *a, const byte *modulus);
 WOLFSSL_LOCAL void fprime_mul(byte *r, const byte *a, const byte *b,
-		                      const byte *modulus);
+                                      const byte *modulus);
 WOLFSSL_LOCAL void fprime_copy(byte *x, const byte *a);
 
 #endif /* CURVE25519_SMALL || ED25519_SMALL */
