@@ -1,6 +1,6 @@
 /* mem_track.h
  *
- * Copyright (C) 2006-2022 wolfSSL Inc.
+ * Copyright (C) 2006-2023 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
@@ -24,8 +24,6 @@
 #ifndef WOLFSSL_MEM_TRACK_H
 #define WOLFSSL_MEM_TRACK_H
 
-#if defined(USE_WOLFSSL_MEMORY) && !defined(WOLFSSL_STATIC_MEMORY)
-
 /* The memory tracker overrides the wolfSSL memory callback system and uses a
  * static to track the total, peak and currently allocated bytes.
  *
@@ -61,20 +59,32 @@
  */
 
 #include "wolfssl/wolfcrypt/settings.h"
+#include "wolfssl/wolfcrypt/types.h"
 #include "wolfssl/wolfcrypt/logging.h"
+#include "wolfssl/wolfcrypt/error-crypt.h"
+#include "wolfssl/wolfcrypt/memory.h"
 
 #if defined(WOLFSSL_TRACK_MEMORY) || defined(HAVE_STACK_SIZE) || \
     defined(HAVE_STACK_SIZE_VERBOSE)
-#include <stdio.h>
-#endif
-
-#if defined(WOLFSSL_TRACK_MEMORY)
-    #define DO_MEM_STATS
-    #if defined(__linux__) || defined(__MACH__)
-        #define DO_MEM_LIST
+    #ifdef NO_STDIO_FILESYSTEM
+        /* if wc_port.h/linuxkm_wc_port.h doesn't define printf, then the user
+         * needs to define it.
+         */
+        #define wc_mem_printf(...) printf(__VA_ARGS__)
+    #else
+        #include <stdio.h>
+        #define wc_mem_printf(...) fprintf(stderr, __VA_ARGS__)
     #endif
 #endif
 
+/* Track Memory */
+#if defined(WOLFSSL_TRACK_MEMORY) && defined(USE_WOLFSSL_MEMORY) && \
+    !defined(WOLFSSL_STATIC_MEMORY)
+
+#define DO_MEM_STATS
+#if (defined(__linux__) && !defined(WOLFSSL_LINUXKM)) || defined(__MACH__)
+    #define DO_MEM_LIST
+#endif
 
 typedef struct memoryStats {
     long totalAllocs;     /* number of allocations */
@@ -84,12 +94,12 @@ typedef struct memoryStats {
     long currentBytes;    /* total current bytes in use */
 #ifdef WOLFSSL_TRACK_MEMORY_VERBOSE
     long peakAllocsTripOdometer; /* peak number of concurrent allocations,
-                                    * subject to reset by
-                                    * wolfCrypt_heap_peak_checkpoint()
-                                    */
+                                  * subject to reset by
+                                  * wolfCrypt_heap_peak_checkpoint()
+                                  */
     long peakBytesTripOdometer; /* peak concurrent bytes, subject to reset
-                                    * by wolfCrypt_heap_peak_checkpoint()
-                                    */
+                                 * by wolfCrypt_heap_peak_checkpoint()
+                                 */
 #endif
 } memoryStats;
 
@@ -110,7 +120,8 @@ typedef struct memHint {
 typedef struct memoryTrack {
     union {
         memHint hint;
-        byte    alignit[sizeof(memHint) + ((16-1) & ~(16-1))]; /* make sure we have strong alignment */
+        /* make sure we have strong alignment */
+        byte    alignit[sizeof(memHint) + ((16-1) & ~(16-1))];
     } u;
 } memoryTrack;
 
@@ -123,7 +134,7 @@ typedef struct memoryList {
 } memoryList;
 #endif
 
-#if defined(WOLFSSL_TRACK_MEMORY)
+
 static memoryStats ourMemStats;
 
 #ifdef DO_MEM_LIST
@@ -131,31 +142,12 @@ static memoryStats ourMemStats;
     static memoryList ourMemList;
     static pthread_mutex_t memLock = PTHREAD_MUTEX_INITIALIZER;
 #endif
-#endif
-
-
-/* if defined to not using inline then declare function prototypes */
-#ifdef NO_INLINE
-    #define WC_STATIC
-    #ifdef WOLFSSL_DEBUG_MEMORY
-            WOLFSSL_LOCAL void* TrackMalloc(size_t sz, const char* func, unsigned int line);
-            WOLFSSL_LOCAL void TrackFree(void* ptr, const char* func, unsigned int line);
-            WOLFSSL_LOCAL void* TrackRealloc(void* ptr, size_t sz, const char* func, unsigned int line);
-    #else
-            WOLFSSL_LOCAL void* TrackMalloc(size_t sz);
-            WOLFSSL_LOCAL void TrackFree(void* ptr);
-            WOLFSSL_LOCAL void* TrackRealloc(void* ptr, size_t sz);
-    #endif
-    WOLFSSL_LOCAL int InitMemoryTracker(void);
-    WOLFSSL_LOCAL void ShowMemoryTracker(void);
-#else
-    #define WC_STATIC static
-#endif
 
 #ifdef WOLFSSL_DEBUG_MEMORY
-WC_STATIC WC_INLINE void* TrackMalloc(size_t sz, const char* func, unsigned int line)
+static WC_INLINE void* TrackMalloc(size_t sz, const char* func,
+                                   unsigned int line)
 #else
-WC_STATIC WC_INLINE void* TrackMalloc(size_t sz)
+static WC_INLINE void* TrackMalloc(size_t sz)
 #endif
 {
     memoryTrack* mt;
@@ -178,7 +170,8 @@ WC_STATIC WC_INLINE void* TrackMalloc(size_t sz)
 
 #ifdef WOLFSSL_DEBUG_MEMORY
 #ifdef WOLFSSL_DEBUG_MEMORY_PRINT
-    fprintf(stderr, "Alloc: %p -> %u at %s:%d\n", header->thisMemory, (word32)sz, func, line);
+    wc_mem_printf("Alloc: %p -> %u at %s:%d\n",
+        header->thisMemory, (word32)sz, func, line);
 #else
     (void)func;
     (void)line;
@@ -189,18 +182,23 @@ WC_STATIC WC_INLINE void* TrackMalloc(size_t sz)
     ourMemStats.totalAllocs++;
     ourMemStats.totalBytes   += sz;
     ourMemStats.currentBytes += sz;
+#ifdef WOLFSSL_TRACK_MEMORY_VERBOSE
+    if (ourMemStats.peakAllocsTripOdometer < ourMemStats.totalAllocs -
+            ourMemStats.totalDeallocs) {
+        ourMemStats.peakAllocsTripOdometer = ourMemStats.totalAllocs -
+            ourMemStats.totalDeallocs;
+    }
+    if (ourMemStats.peakBytesTripOdometer < ourMemStats.currentBytes)
+#endif
+    {
     #ifdef WOLFSSL_TRACK_MEMORY_VERBOSE
-    if (ourMemStats.peakAllocsTripOdometer < ourMemStats.totalAllocs - ourMemStats.totalDeallocs)
-        ourMemStats.peakAllocsTripOdometer = ourMemStats.totalAllocs - ourMemStats.totalDeallocs;
-    if (ourMemStats.peakBytesTripOdometer < ourMemStats.currentBytes) {
         ourMemStats.peakBytesTripOdometer = ourMemStats.currentBytes;
     #endif
         if (ourMemStats.currentBytes > ourMemStats.peakBytes)
             ourMemStats.peakBytes = ourMemStats.currentBytes;
-    #ifdef WOLFSSL_TRACK_MEMORY_VERBOSE
     }
-    #endif
-#endif
+#endif /* DO_MEM_STATS */
+
 #ifdef DO_MEM_LIST
     if (pthread_mutex_lock(&memLock) == 0) {
     #ifdef WOLFSSL_DEBUG_MEMORY
@@ -223,16 +221,16 @@ WC_STATIC WC_INLINE void* TrackMalloc(size_t sz)
 
         pthread_mutex_unlock(&memLock);
     }
-#endif
+#endif /* DO_MEM_LIST */
 
     return header->thisMemory;
 }
 
 
 #ifdef WOLFSSL_DEBUG_MEMORY
-WC_STATIC WC_INLINE void TrackFree(void* ptr, const char* func, unsigned int line)
+static WC_INLINE void TrackFree(void* ptr, const char* func, unsigned int line)
 #else
-WC_STATIC WC_INLINE void TrackFree(void* ptr)
+static WC_INLINE void TrackFree(void* ptr)
 #endif
 {
     memoryTrack* mt;
@@ -286,7 +284,7 @@ WC_STATIC WC_INLINE void TrackFree(void* ptr)
 
 #ifdef WOLFSSL_DEBUG_MEMORY
 #ifdef WOLFSSL_DEBUG_MEMORY_PRINT
-    fprintf(stderr, "Free: %p -> %u at %s:%d\n", ptr, (word32)sz, func, line);
+    wc_mem_printf("Free: %p -> %u at %s:%d\n", ptr, (word32)sz, func, line);
 #else
     (void)func;
     (void)line;
@@ -303,9 +301,10 @@ WC_STATIC WC_INLINE void TrackFree(void* ptr)
 
 
 #ifdef WOLFSSL_DEBUG_MEMORY
-WC_STATIC WC_INLINE void* TrackRealloc(void* ptr, size_t sz, const char* func, unsigned int line)
+static WC_INLINE void* TrackRealloc(void* ptr, size_t sz, const char* func,
+                                    unsigned int line)
 #else
-WC_STATIC WC_INLINE void* TrackRealloc(void* ptr, size_t sz)
+static WC_INLINE void* TrackRealloc(void* ptr, size_t sz)
 #endif
 {
 #ifdef WOLFSSL_DEBUG_MEMORY
@@ -340,98 +339,95 @@ WC_STATIC WC_INLINE void* TrackRealloc(void* ptr, size_t sz)
     return ret;
 }
 
-#ifdef WOLFSSL_TRACK_MEMORY
 static wolfSSL_Malloc_cb mfDefault = NULL;
 static wolfSSL_Free_cb ffDefault = NULL;
 static wolfSSL_Realloc_cb rfDefault = NULL;
 
-WC_STATIC WC_INLINE int InitMemoryTracker(void)
+static WC_INLINE int InitMemoryTracker(void)
 {
     int ret;
 
     ret = wolfSSL_GetAllocators(&mfDefault, &ffDefault, &rfDefault);
     if (ret < 0) {
-        fprintf(stderr, "wolfSSL GetAllocators failed to get the defaults\n");
+        wc_mem_printf("wolfSSL GetAllocators failed to get the defaults\n");
     }
     ret = wolfSSL_SetAllocators(TrackMalloc, TrackFree, TrackRealloc);
     if (ret < 0) {
-        fprintf(stderr, "wolfSSL SetAllocators failed for track memory\n");
+        wc_mem_printf("wolfSSL SetAllocators failed for track memory\n");
         return ret;
     }
 
 #ifdef DO_MEM_LIST
     if (pthread_mutex_lock(&memLock) == 0)
+#endif
     {
-#endif
+    #ifdef DO_MEM_STATS
+        ourMemStats.totalAllocs  = 0;
+        ourMemStats.totalDeallocs = 0;
+        ourMemStats.totalBytes   = 0;
+        ourMemStats.peakBytes    = 0;
+        ourMemStats.currentBytes = 0;
+    #ifdef WOLFSSL_TRACK_MEMORY_VERBOSE
+        ourMemStats.peakAllocsTripOdometer = 0;
+        ourMemStats.peakBytesTripOdometer    = 0;
+    #endif
+    #endif /* DO_MEM_STATS */
 
-#ifdef DO_MEM_STATS
-    ourMemStats.totalAllocs  = 0;
-    ourMemStats.totalDeallocs = 0;
-    ourMemStats.totalBytes   = 0;
-    ourMemStats.peakBytes    = 0;
-    ourMemStats.currentBytes = 0;
-#ifdef WOLFSSL_TRACK_MEMORY_VERBOSE
-    ourMemStats.peakAllocsTripOdometer = 0;
-    ourMemStats.peakBytesTripOdometer    = 0;
-#endif
-#endif
+    #ifdef DO_MEM_LIST
+        XMEMSET(&ourMemList, 0, sizeof(ourMemList));
 
-#ifdef DO_MEM_LIST
-    XMEMSET(&ourMemList, 0, sizeof(ourMemList));
-
-    pthread_mutex_unlock(&memLock);
+        pthread_mutex_unlock(&memLock);
+    #endif
     }
-#endif
 
     return ret;
 }
 
-WC_STATIC WC_INLINE void ShowMemoryTracker(void)
+static WC_INLINE void ShowMemoryTracker(void)
 {
 #ifdef DO_MEM_LIST
     if (pthread_mutex_lock(&memLock) == 0)
+#endif
     {
-#endif
+    #ifdef DO_MEM_STATS
+        wc_mem_printf("total   Allocs   = %9ld\n", ourMemStats.totalAllocs);
+        wc_mem_printf("total   Deallocs = %9ld\n", ourMemStats.totalDeallocs);
+        wc_mem_printf("total   Bytes    = %9ld\n", ourMemStats.totalBytes);
+        wc_mem_printf("peak    Bytes    = %9ld\n", ourMemStats.peakBytes);
+        wc_mem_printf("current Bytes    = %9ld\n", ourMemStats.currentBytes);
+    #endif
 
-#ifdef DO_MEM_STATS
-    fprintf(stderr, "total   Allocs   = %9ld\n", ourMemStats.totalAllocs);
-    fprintf(stderr, "total   Deallocs = %9ld\n", ourMemStats.totalDeallocs);
-    fprintf(stderr, "total   Bytes    = %9ld\n", ourMemStats.totalBytes);
-    fprintf(stderr, "peak    Bytes    = %9ld\n", ourMemStats.peakBytes);
-    fprintf(stderr, "current Bytes    = %9ld\n", ourMemStats.currentBytes);
-#endif
-
-#ifdef DO_MEM_LIST
-    if (ourMemList.count > 0) {
-        /* print list of allocations */
-        memHint* header;
-        for (header = ourMemList.head; header != NULL; header = header->next) {
-            fprintf(stderr, "Leak: Ptr %p, Size %u"
-            #ifdef WOLFSSL_DEBUG_MEMORY
-                ", Func %s, Line %d"
-            #endif
-                "\n",
-                (byte*)header + sizeof(memHint), (unsigned int)header->thisSize
-            #ifdef WOLFSSL_DEBUG_MEMORY
-                , header->func, header->line
-            #endif
-            );
+    #ifdef DO_MEM_LIST
+        if (ourMemList.count > 0) {
+            /* print list of allocations */
+            memHint* header;
+            for (header = ourMemList.head;
+                 header != NULL;
+                 header = header->next) {
+        #ifdef WOLFSSL_DEBUG_MEMORY
+                wc_mem_printf("Leak: Ptr %p, Size %u, Func %s, Line %d\n",
+                    (byte*)header + sizeof(memHint),
+                    (unsigned int)header->thisSize, header->func, header->line);
+        #else
+                wc_mem_printf("Leak: Ptr %p, Size %u\n",
+                    (byte*)header + sizeof(memHint),
+                    (unsigned int)header->thisSize);
+        #endif
+            }
         }
-    }
 
-    pthread_mutex_unlock(&memLock);
+        pthread_mutex_unlock(&memLock);
+    #endif
     }
-#endif
 }
 
-WC_STATIC WC_INLINE int CleanupMemoryTracker(void)
+static WC_INLINE int CleanupMemoryTracker(void)
 {
     /* restore default allocators */
     return wolfSSL_SetAllocators(mfDefault, ffDefault, rfDefault);
 }
-#endif /* WOLFSSL_TRACK_MEMORY */
-
-#endif /* USE_WOLFSSL_MEMORY && !WOLFSSL_STATIC_MEMORY */
+#endif /* WOLFSSL_TRACK_MEMORY && USE_WOLFSSL_MEMORY && \
+          !WOLFSSL_STATIC_MEMORY */
 
 
 #ifdef HAVE_STACK_SIZE
@@ -442,10 +438,6 @@ WC_STATIC WC_INLINE int CleanupMemoryTracker(void)
 #include <sched.h>
 #include <unistd.h>
 
-#include <wolfssl/wolfcrypt/types.h>
-#include <wolfssl/wolfcrypt/logging.h>
-#include <wolfssl/wolfcrypt/error-crypt.h>
-
 typedef void* (*thread_func)(void* args);
 #define STACK_CHECK_VAL 0x01
 
@@ -555,7 +547,7 @@ int StackSizeHWMReset(void)
 
 #define STACK_SIZE_CHECKPOINT_MSG(msg) ({             \
     ssize_t HWM = StackSizeHWM_OffsetCorrected();     \
-    fprintf(stderr, "%ld\t%s\n", (long int)HWM, msg); \
+    wc_mem_printf("%ld\t%s\n", (long int)HWM, msg);   \
     StackSizeHWMReset();                              \
     })
 
@@ -566,8 +558,9 @@ int StackSizeHWMReset(void)
     printf("    relative stack peak usage = %ld bytes\n", (long int)HWM);  \
     _ret = StackSizeHWMReset();                      \
     if ((max >= 0) && (HWM > (ssize_t)(max))) {      \
-        fprintf(stderr,                              \
-            "    relative stack usage at %s L%d exceeds designated max %ld bytes.\n", \
+        wc_mem_printf(                               \
+            "    relative stack usage at %s L%d exceeds designated " \
+                "max %ld bytes.\n", \
             __FILE__, __LINE__, (long int)(max));    \
         _ret = -1;                                   \
     }                                                \
@@ -602,7 +595,7 @@ static WC_INLINE int StackSizeCheck(struct func_args* args, thread_func tf)
 
     ret = posix_memalign((void**)&myStack, sysconf(_SC_PAGESIZE), stackSize);
     if (ret != 0 || myStack == NULL) {
-        fprintf(stderr, "posix_memalign failed\n");
+        wc_mem_printf("posix_memalign failed\n");
         return -1;
     }
 
@@ -610,13 +603,13 @@ static WC_INLINE int StackSizeCheck(struct func_args* args, thread_func tf)
 
     ret = pthread_attr_init(&myAttr);
     if (ret != 0) {
-        fprintf(stderr, "attr_init failed\n");
+        wc_mem_printf("attr_init failed\n");
         return ret;
     }
 
     ret = pthread_attr_setstack(&myAttr, myStack, stackSize);
     if (ret != 0) {
-        fprintf(stderr, "attr_setstackaddr failed\n");
+        wc_mem_printf("attr_setstackaddr failed\n");
         return ret;
     }
 
@@ -640,7 +633,7 @@ static WC_INLINE int StackSizeCheck(struct func_args* args, thread_func tf)
 
     ret = pthread_join(threadId, &status);
     if (ret != 0) {
-        fprintf(stderr, "pthread_join failed\n");
+        wc_mem_printf("pthread_join failed\n");
         return ret;
     }
 
@@ -689,7 +682,7 @@ static WC_INLINE int StackSizeCheck_launch(struct func_args* args,
 
     ret = posix_memalign((void**)&myStack, sysconf(_SC_PAGESIZE), stackSize);
     if (ret != 0 || myStack == NULL) {
-        fprintf(stderr, "posix_memalign failed\n");
+        wc_mem_printf("posix_memalign failed\n");
         free(shim_args);
         return -1;
     }
@@ -698,7 +691,7 @@ static WC_INLINE int StackSizeCheck_launch(struct func_args* args,
 
     ret = pthread_attr_init(&myAttr);
     if (ret != 0) {
-        fprintf(stderr, "attr_init failed\n");
+        wc_mem_printf("attr_init failed\n");
         free(shim_args);
         free(myStack);
         return ret;
@@ -706,7 +699,7 @@ static WC_INLINE int StackSizeCheck_launch(struct func_args* args,
 
     ret = pthread_attr_setstack(&myAttr, myStack, stackSize);
     if (ret != 0) {
-        fprintf(stderr, "attr_setstackaddr failed\n");
+        wc_mem_printf("attr_setstackaddr failed\n");
     }
 
     shim_args->myStack = myStack;
@@ -730,7 +723,8 @@ static WC_INLINE int StackSizeCheck_launch(struct func_args* args,
     return 0;
 }
 
-static WC_INLINE int StackSizeCheck_reap(pthread_t threadId, void *stack_context)
+static WC_INLINE int StackSizeCheck_reap(pthread_t threadId,
+    void *stack_context)
 {
     struct stack_size_debug_context *shim_args =
         (struct stack_size_debug_context *)stack_context;
@@ -738,7 +732,7 @@ static WC_INLINE int StackSizeCheck_reap(pthread_t threadId, void *stack_context
     void *status;
     int ret = pthread_join(threadId, &status);
     if (ret != 0) {
-        fprintf(stderr, "pthread_join failed\n");
+        wc_mem_printf("pthread_join failed\n");
         return ret;
     }
 
@@ -765,7 +759,6 @@ static WC_INLINE int StackSizeCheck_reap(pthread_t threadId, void *stack_context
     return (int)((size_t)status);
 }
 
-
 #endif /* HAVE_STACK_SIZE */
 
 
@@ -787,12 +780,12 @@ static WC_INLINE void StackTrap(void)
 {
     struct rlimit  rl;
     if (getrlimit(RLIMIT_STACK, &rl) != 0) {
-        fprintf(stderr, "getrlimit failed\n");
+        wc_mem_printf("getrlimit failed\n");
     }
     printf("rlim_cur = %llu\n", rl.rlim_cur);
     rl.rlim_cur = 1024*21;  /* adjust trap size here */
     if (setrlimit(RLIMIT_STACK, &rl) != 0) {
-        fprintf(stderr, "setrlimit failed\n");
+        wc_mem_printf("setrlimit failed\n");
     }
 }
 
@@ -809,13 +802,13 @@ static WC_INLINE void StackTrap(void)
 #define STACK_SIZE_CHECKPOINT(...) (__VA_ARGS__)
 #endif
 #ifndef STACK_SIZE_CHECKPOINT_MSG
-#define STACK_SIZE_CHECKPOINT_MSG(msg)
+#define STACK_SIZE_CHECKPOINT_MSG(msg) WC_DO_NOTHING
 #endif
 #ifndef STACK_SIZE_CHECKPOINT_WITH_MAX_CHECK
 #define STACK_SIZE_CHECKPOINT_WITH_MAX_CHECK(max, ...) (__VA_ARGS__, 0)
 #endif
 #ifndef STACK_SIZE_INIT
-#define STACK_SIZE_INIT()
+#define STACK_SIZE_INIT() WC_DO_NOTHING
 #endif
 
 #endif /* WOLFSSL_MEM_TRACK_H */
