@@ -1,6 +1,6 @@
 /* wolfmath.h
  *
- * Copyright (C) 2006-2022 wolfSSL Inc.
+ * Copyright (C) 2006-2023 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
@@ -31,12 +31,27 @@ This library provides big integer math functions.
     extern "C" {
 #endif
 
+#include <wolfssl/wolfcrypt/types.h>
+
 #ifdef WOLFSSL_PUBLIC_MP
     #define MP_API   WOLFSSL_API
 #else
     #define MP_API   WOLFSSL_LOCAL
 #endif
 
+
+#if defined(USE_FAST_MATH)
+    #include <wolfssl/wolfcrypt/tfm.h>
+#elif defined(USE_INTEGER_HEAP_MATH)
+    #include <wolfssl/wolfcrypt/integer.h>
+#else
+    #include <wolfssl/wolfcrypt/sp_int.h>
+#endif
+
+#if !defined(NO_BIG_INT) || defined(WOLFSSL_SP_MATH)
+    #include <wolfssl/wolfcrypt/random.h>
+#endif
+
 #ifndef MIN
    #define MIN(x,y) ((x)<(y)?(x):(y))
 #endif
@@ -53,7 +68,7 @@ This library provides big integer math functions.
     extern const wc_ptr_t wc_off_on_addr[2];
 #endif
 
-
+#if !defined(NO_BIG_INT) || defined(WOLFSSL_SP_MATH)
 /* common math functions */
 MP_API int get_digit_count(const mp_int* a);
 MP_API mp_digit get_digit(const mp_int* a, int n);
@@ -62,15 +77,18 @@ WOLFSSL_LOCAL void mp_reverse(unsigned char *s, int len);
 
 WOLFSSL_API int mp_cond_copy(mp_int* a, int copy, mp_int* b);
 WOLFSSL_API int mp_rand(mp_int* a, int digits, WC_RNG* rng);
+#endif
 
 #define WC_TYPE_HEX_STR 1
 #define WC_TYPE_UNSIGNED_BIN 2
-#if defined(WOLFSSL_QNX_CAAM)
+#if defined(WOLFSSL_QNX_CAAM) || defined(WOLFSSL_IMXRT1170_CAAM)
     #define WC_TYPE_BLACK_KEY 3
 #endif
 
+#if defined(HAVE_ECC) || defined(WOLFSSL_EXPORT_INT)
 WOLFSSL_API int wc_export_int(mp_int* mp, byte* buf, word32* len,
     word32 keySz, int encType);
+#endif
 
 #ifdef HAVE_WOLF_BIGINT
     #if !defined(WOLF_BIGINT_DEFINED)
@@ -96,6 +114,10 @@ WOLFSSL_API int wc_export_int(mp_int* mp, byte* buf, word32* len,
 #endif /* HAVE_WOLF_BIGINT */
 
 
+#ifdef HAVE_WC_INTROSPECTION
+    WOLFSSL_API const char *wc_GetMathInfo(void);
+#endif
+
 #ifdef __cplusplus
     } /* extern "C" */
 #endif
