@@ -1,6 +1,6 @@
 /* wolfSSL.cs
  *
- * Copyright (C) 2006-2021 wolfSSL Inc.
+ * Copyright (C) 2006-2023 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
@@ -19,6 +19,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335, USA
  */
 
+
 using System;
 using System.Runtime.InteropServices;
 using System.Text;
@@ -724,7 +725,7 @@ namespace wolfSSL.CSharp {
                 int ret;
                 byte[] msg;
 
-                buf.Clear(); /* Clear incomming buffer */
+                buf.Clear(); /* Clear incoming buffer */
 
                 if (sslCtx == IntPtr.Zero)
                 {
@@ -1756,10 +1757,10 @@ namespace wolfSSL.CSharp {
         }
 
         /// <summary>
-        /// Used to load in the private key from a file 
+        /// Used to load in the private key from a file
         /// </summary>
         /// <param name="ctx">CTX structure for TLS/SSL connections </param>
-        /// <param name="fileKey">Name of the file, includeing absolute directory</param>
+        /// <param name="fileKey">Name of the file, including absolute directory</param>
         /// <param name="type">Type of file ie PEM or DER</param>
         /// <returns>1 on success</returns>
         public static int CTX_use_PrivateKey_file(IntPtr ctx, string fileKey, int type)
@@ -1997,7 +1998,7 @@ namespace wolfSSL.CSharp {
                     wolfSSL_sk_X509_free(sk);
                 }
                 return ret;
-                
+
             }
             catch (Exception e)
             {
